<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/

/* File-desc: class for userhandling */
	namespace admin;
	use \db\fs\FsHandler;
	class UserAdmin{
		/* ### Member vars ### */
			private $mDbConn;
		
		
		/* ### Constructors ### */
			function __construct(\PDO $aDbConn){
				$this->setDbConn($aDbConn);
			}
		
		/* ### Public Methods ### */
			/**
			 * @param String $aUserName
			 * @param String $aPwd
			 * @return on error boolean false else int userId
			 */
			public function checkLogin($aUserName,$aPwd){
				$aPwd = md5($aPwd); //Create md5 hash				
				/*** Prepare the sql statement and bind the parameters ***/
					$stmt = $this->getDbConn()->prepare("SELECT id from users WHERE name = :username AND password = :pwd LIMIT 1");
					$stmt->bindParam(':username',$aUserName);
					$stmt->bindParam(':pwd',$aPwd);
				/*** Execute ***/
					$stmt->execute();
				/*** Check and return ***/
						$user = $stmt->fetch();
						if(!is_array($user)){return false;}
						return $user['id'];
			}
		
		
			/**
			 * @param String $aUserName
			 * @param String $aPwd
			 * @param db\DbFs $aDbFsMod
			 * @return boolean
			 */
			public function addUser($aUserName,$aPwd,$aAdmin=0){
				/*** Check username ***/
				if($this->userNameExists($aUserName)){
					error_log("User: ".$aUserName." already exists");
					return false;
				}
				
				
				$fsHandler = new FsHandler($this->getDbConn(),0);
				/*** Prepare sql statement ***/
				$query = "INSERT INTO users (admin,name,password,user_space) VALUES(?,?,?,?)";
				$stmt = $this->getDbConn()->prepare($query);
				if($stmt->execute(Array($aAdmin,$aUserName,md5($aPwd),$fsHandler->getDefaultUserSpace()))){
					$stmt = $this->getDbConn()->prepare("SELECT id FROM users WHERE  name = :name AND password = :pwd");
					$stmt->bindParam(":name",$aUserName);$stmt->bindParam(":pwd",md5($aPwd));
					$stmt->execute();
					$res = $stmt->fetch();
					$fsHandler = new \db\fs\FsHandler($this->getDbConn(),$res['id']);
					$fsHandler->createNewUserFs();
					return true;
				}else{
					return false;
				}				
			}	

			//-------------------------------------
			
			public function isAdmin($aUserId){
				$stmt = $this->getDbConn()->prepare("SELECT admin From users WHERE id = ?");
				$stmt->execute(Array($aUserId));
				$res = $stmt->fetch();
				if ($res['admin']=="1"){
					return true;
				}else{
					return false;
				}
			}
			
			//--------------------------------------
			
			public function getUserName($aUserId){
				$stmt = $this->getDbConn()->prepare("SELECT name From users WHERE id = ?");
				$stmt->execute(Array($aUserId));
				$res = $stmt->fetch();
				return $res['name'];
			}
			
			//---------------------------------------
			
			public function getWebInterfaceDesign($aUserId){
				$stmt = $this->getDbConn()->prepare("SELECT webinterface_design From users WHERE id = ?");
				$stmt->execute(Array($aUserId));
				$res = $stmt->fetch();
				return $res['webinterface_design'];
			}
		/* ### Private Methods ### */	
			private function userNameExists($userName){
				$stmt = $this->getDbConn()->prepare("SELECT COUNT(*) FROM users WHERE name = :name");
				$stmt->bindParam(":name",$userName);
				$stmt->execute();
				if($stmt->fetchColumn()!=false){return true;}else{return false;}
			}
		/* ### Setters and Getters ### */
			private function setDbConn($aMod){$this->mDbConn = $aMod;}
			private function getDbConn(){return $this->mDbConn;}		
		
	}