<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/

	/* File-desc: server <-> desktop-client communication interface */
	namespace comInterface;
	use errorHandler;
	
	include_once("admin/UserAdmin.inc");
	include_once("db/fs/FsHandler.inc");
	use admin,db\fs;
	
	class ComInterface{
		/* ### Member Vars ### */
			private $mUserId;
			private $mDbConn;
			/**
			 * @var FsHandler
			 */
			private $mFsHandler;
			
			private $mLoggedIn;
		/* ### Constructors ### */
			function __construct(\PDO $aDbConn){
				$this->setDbConn($aDbConn);
				if(!$this->checkLogin()){
					header('eversync-denied: true'); 
					return;
				}else{
					$this->setFsHandler(new \db\fs\FsHandler($this->getDbConn(),$this->getUserId()));
					$this->handleRequests();
				}
			}					
		/* ### Public Methods ### */
		
		
		/* ### Private Methods ### */
			
			private function handleRequests(){
				if(isset($_GET['action'])){
						switch($_GET['action']){
							case "getFsStructure":
								$path ="";
								$time =0;
								if(isset($_GET['folder'])){ $path=$_GET['folder'];}
								if(isset($_GET['time'])){$time = $_GET['time'];}
								$this->getFileList($path,$time,true);
								break;
							case "getEntries":
								$path ="";
								$time =0;
								if(isset($_GET['folder'])){ $path=$_GET['folder'];}
								$this->getFileList($path,$time,false);
								break;
							case "getChangesCount":
								$time = 0;
								if(isset($_GET['time'])){
									echo count($this->getFileList("/",$time,true));
								}
								break;
							case "checkUser":
								$this->checkLogin();
								break;
							case "getFile":
								if(isset($_GET['path']) && isset($_GET['name'])){
									$path=$_GET['path'];
									$name = $_GET['name'];
									echo $this->getFile($path,$name);
								}
								break;
							case "updateFile":
								if(isset($_GET['path']) && isset($_GET['name']) && isset($_FILES['file'])){
									if($_FILES['file']['error']=="0"){
										$lastChange = 0;
										if(isset($_GET['last_change'])){$lastChange = $_GET['last_change'];}									
										$this->getFsHandler()->updateFile($_GET['path'],$_GET['name'],basename($_FILES['file']['tmp_name']),$_FILES['type'],$lastChange);
									}else{
										echo 'File upload error: '.$_FILES['file']['error'].'. For more infos look at: http://www.php.net/manual/en/features.file-upload.errors.php';
									}
								}
								break;
							case "addFsEntry":
								if(isset($_GET['path']) && isset($_GET['name']) && isset($_GET['type'])){									
									$lastChange = 0;
									if(isset($_GET['last_change'])){$lastChange = $_GET['last_change'];}									
									if($_GET['type']==1){ //File										
										if(isset($_FILES['file'])){
											if($_FILES['file']['error']=="0"){
												$this->getFsHandler()->addFile($_GET['path'],$_GET['name'],basename($_FILES['file']['tmp_name']),$lastChange);
											}else{
												echo 'File upload error: '.$_FILES['file']['error'].'. For more infos look at: http://www.php.net/manual/en/features.file-upload.errors.php';
											}
										}else{
											$this->getFsHandler()->addFile($_GET['path'],$_GET['name'],null,$lastChange,true,true);
										}
									}else{ //Folder
										$this->getFsHandler()->addFolder($_GET['path'],$_GET['name'],$lastChange);		
									}
								}
								break;
							case "deleteFsEntry":
								if(isset($_GET['path']) && isset($_GET['name'])){
									$this->getFsHandler()->deleteFsEntry($_GET['path'],$_GET['name']);
								}
								break;
							case "updateFsEntry":
								if(isset($_GET['path']) && isset($_GET['name'])){
									$lastChange = 0;
									if(isset($_GET['last_change'])){$lastChange = $_GET['last_change'];}
									$this->getFsHandler()->setLastChange($_GET['path'],$_GET['name'],$lastChange);
								}
								break;
							case "fileHashExists":
								if(isset($_GET['hash']) && $_GET['hash']!=""){
									if($this->getFsHandler()->fileExists($_GET['hash'])){
										echo "true";
									}else{
										echo "false";
									}
								}
								break;
							default:
								assert(false);
						}
				}
				
				if(isset($_GET['debug_req'])){
					switch($_GET['debug_req']){
						case "mainTest":
							$this->addUser("test","test");
							$this->checkLogin();
							$this->setFsHandler(new \db\fs\FsHandler($this->getDbConn(),$this->getUserId())); //Because the userid has to be set							
							$this->folderTest();
							$this->fileTest();
							//$this->fsListTest();
							echo "\nAll tests completed!\n";
							break;
						default:
							assert(false);
					}
				}
			}
			
			//-----------------------------
			
			private function fsListTest(){
				$this->getFileList("/",0);
			}
			
			//-----------------------------
			
			private function folderTest(){				
				try{
					$this->getFsHandler()->addFolder("/","testFolder");
					$this->getFsHandler()->addFolder("/testFolder/","testFolder");					
					$this->getFsHandler()->addFolder("/testFolder/testFolder/","testFolder");		
					$this->getFsHandler()->renameFsEntry("/","testFolder","newFolder"); 
				}catch(\Exception $e){
					\errorHandler\ErrorHandler::showException($e);
				}catch(PDOException $e){
					\errorHandler\ErrorHandler::showException($e);
				}	
				$this->getFsHandler()->deleteFsEntry("/","newFolder");				
			}
			
			//-----------------------------
			
			private function fileTest(){
				
					try{
						$this->getFsHandler()->addFile("/","oha.png","oha.png",0,false,true);
						$this->getFsHandler()->addFile("/","oha2.png","oha2.png",0,false,true);
					}catch(\Exception $e){
						\errorHandler\ErrorHandler::showException($e);
					}
					try{
						$file = $this->getFsHandler()->getFile("/","oha.png");
						$file = $this->getFsHandler()->getFile("/","oha2.png");
						//echo $file;
					}catch(\Exception $e){
						\errorHandler\ErrorHandler::showException($e);
					}
					
					$this->getFsHandler()->deleteFsEntry("/","oha.png");
					$this->getFsHandler()->deleteFsEntry("/","oha2.png");
			}
			
			//-----------------------------
			
			private function getFileList($aPath,$aTime,$aRecursive=true){
				$fsHandler = new \db\fs\FsHandler($this->getDbConn(),$this->getUserId());
				$fsArr = $fsHandler->getFsList($aPath,$aTime,$aRecursive);
				$re="";
				
				$lineSeperator= chr(30); // ^^
				$paraMeterSeperator = chr(31); //^_
				foreach($fsArr as $entry){
					$re.=($entry->getId().$paraMeterSeperator.$entry->getType().$paraMeterSeperator.$entry->getUpperFolder().$paraMeterSeperator.$entry->getPath().$paraMeterSeperator
					.$entry->getName().$paraMeterSeperator.$entry->getLastChange().$paraMeterSeperator.$entry->getHash().$lineSeperator);
				}
				/*** Remove last # ***/
				$re=substr($re,0,strlen($re)-strlen($lineSeperator));				
				echo $re;
			}
			
			//-----------------------------
			
			private function getFile($aPath,$aName){
				$fsHandler = new \db\fs\FsHandler($this->getDbConn(),$this->getUserId());
				return $fsHandler->getFile($aPath,$aName);
			}
			
			//-----------------------------
			
			private function adduser($aUserName,$aPwd){
				$userAdmin = new admin\UserAdmin($this->getDbConn());
				$userAdmin->addUser($aUserName,$aPwd);	
			}			
			
			//-----------------------------
			
			private function checkLogin(){
				$userAdmin = new admin\UserAdmin($this->getDbConn());
				if(!isset($_GET['uname']) && !isset($_GET['pwd'])){return false;}
				$userId = $userAdmin->checkLogin($_GET['uname'],$_GET['pwd']);
				if($userId===false){
					$this->setLoggedIn(false);
					return false;
				}else{
					$this->setLoggedIn(true);
					$this->setUserId($userId);
					return true;	
				}				
			}
		
		/* ### Setters and Getters ### */
			private function setUserId($aId){$this->mUserId=$aId;}
			private function getUserId(){return $this->mUserId;}
			
			private function setDbConn($aMod){$this->mDbConn = $aMod;}
			private function getDbConn(){return $this->mDbConn;}
			
			private function setFsHandler($aValue){$this->mFsHandler =	$aValue;}			
			private function getFsHandler(){return $this->mFsHandler;}
			
			private function setLoggedIn($aVal){$this->mLoggedIn = $aVal;}
			private function isLoggedIn(){return $this->mLoggedIn;}
	}
?>