<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/

	/* File-desc: Db-Core File. Handles the db-connections and actions */
/*** Headinfos ***/
	namespace db;  
	include_once("db/fs/FsHandler.inc");
	include_once("errorHandler/ErrorHandler.inc");
/*** Class ***/
class Database{
	/* Member Vars */
		private $mDbConnection;
			
		private $mTblArr;	
		private $mIndexArr;
	/* Constructor */
		function __construct(){
			$this->setTblArr(Array("config","users","filesystem","files","history"));		
			$this->setIndexArr(Array("hash_index","file_hash_index","history_id_index","history_time_index"));
			$this->connectToDb();			
		}
			
	
	/* Destruct */
		function __destruct(){
			$this->setDbConnection(null);
		}
		
	/* Public methods */
		public function clearDb(){
				/** Drop tables **/
	   				 try{
	   				 	foreach($this->getTblArr() as $tbl){
	   				 		if($this->getDbConnection()->exec("DROP TABLE IF EXISTS $tbl")===false){return false;}
	   				 	}	
	   				 	foreach($this->getIndexArr() as $idx){
	   				 		if($this->getDbConnection()->exec("DROP INDEX IF EXISTS $idx")===false){return false;}
	   				 	}	
	   				 }catch(\PDOException $e){
	   				 	die($e->getMessage());
	   				 }		   				 
	   			/** recreate the tables **/	   				 
	   				 $this->createDbTables();
	   				 
	   		return true;
		}
		
		
	/* Private Methods */
		
		/**
		 * 
		 * @connects to the database
		 * 
		 * @access private
		 * 
		 */
		private function connectToDb(){
			try{
				/*** connect to the database ***/
					$pdo  = new \PDO("sqlite:files/db.sq3");
					$this->setDbConnection($pdo);
    			/*** set the error reporting attribute ***/
    				$this->getDbConnection()->setAttribute(\PDO::ATTR_ERRMODE, \PDO::ERRMODE_EXCEPTION);						
				/*** check the database ***/
					if(!$this->checkDb()){
						try{
							/*** create the tables ***/
								$this->createDbTables();	
								error_log("Created a new set of tables!");
						}catch(Exception $e){
								\errorHandler\ErrorHandler::showException($e);
								die();
						}
					}	
			}catch(\PDOException $e){
				\errorHandler\ErrorHandler::showException($e);
				die();				
			}						
		}
		//----------------------------
		
		/**
		 * 
		 * @checks databases tables of existence
		 * 
		 * @access private
		 * 
		 * @return string|string
		 * 
		 */
		private function checkDb(){						
			/*** Prepare query ***/
				$stmt = $this->getDbConnection()->prepare("SELECT * FROM sqlite_master WHERE type = 'table' AND name = :tblName");
    		/*** bind the paramaters ***/
				$tbl = "";
   			 	$stmt->bindParam(':tblName', $tbl);
			/*** check ***/
   				 try{
   				 	foreach($this->getTblArr() as $tbl){
   				 		$stmt->execute();
   				 		if($stmt->fetch()!=true){return false;}
   				 	}	
   				 }catch(\PDOException $e){
   				 	die($e->getMessage());
   				 }		
    		return true;
		}
		
		//----------------------------
		
		/**
		 * 
		 * @creates the database tables
		 * 
		 * @access private
		 * 
		 */
		private function createDbTables() {				
			if($this->getDbConnection()->exec('CREATE TABLE IF NOT EXISTS config (default_max_user_space int)')===false){					
				throw new \Exception("Can't create table: config");
			}else{
				//Insert Default Config
				$sqlQuery="INSERT INTO config (default_max_user_space) VALUES(?)";
				$stmt = $this->getDbConnection()->prepare($sqlQuery);
				$stmt->execute(Array(1000));
			}
			
			if($this->getDbConnection()->exec('CREATE TABLE IF NOT EXISTS users (
				id INTEGER PRIMARY KEY AUTOINCREMENT,
				admin TINYINT,
				name VARCHAR(255),
				password VARCHAR(32),
				user_space INT,
				webinterface_design VARCHAR(255))')===false){
				throw new \Exception("Can't create table: users");
			}
			
			if($this->getDbConnection()->exec('CREATE TABLE IF NOT EXISTS filesystem(
				id INTEGER PRIMARY KEY AUTOINCREMENT,
				owner INT,
				path VARCHAR(255),
				name VARCHAR(255),
				upper_folder INT,				
				last_change LONG,
				last_update LONG,
				type TINYINT,
				hash VARCHAR(32))')===false){
				throw new \Exception("Can't create table: filesystem");
			}	

			
			if($this->getDbConnection()->exec('CREATE INDEX hash_index ON filesystem(hash)'));
		
			if($this->getDbConnection()->exec('CREATE TABLE IF NOT EXISTS files(
				hash VARCHAR(32) PRIMARY KEY,
				file BLOB,
				type VARCHAR(255),
				size LONG)')===false){
				throw new \Exception("Can't create table: files");
			}else{
				$this->getDbConnection()->exec('CREATE INDEX file_hash_index ON files(hash)');
			}
			
			if($this->getDbConnection()->exec('CREATE TABLE IF NOT EXISTS history(
				id INTEGER PRIMARY KEY AUTOINCREMENT,
				msg VARCHAR(255),
				time LONG,
				type INTEGER,
				userId INTEGER)')===false){
				throw new \Exception("Can't create table: history");
			}else{
				$this->getDbConnection()->exec('CREATE INDEX history_id_index ON history(userId)');
				$this->getDbConnection()->exec('CREATE INDEX history_time_index ON history(time)');
			}
		}
		
	
	/* Setters and Getters */
		private function setDbConnection($aConn){$this->mDbConnection = $aConn;}
		public function getDbConnection(){return $this->mDbConnection;}
		
		private function setTblArr($aArr){$this->mTblArr = $aArr;}
		private function getTblArr(){return $this->mTblArr;}
		
		private function setIndexArr($aArr){$this->mIndexArr = $aArr;}
		private function getIndexArr(){return $this->mIndexArr;}
}