<?php 
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/

	/* File-desc: Handles the saved files in the Db */	
	namespace db\fs;
	
	class FileHandler{
		/* ### Consts ### */
			const cCACHE_FOLDER = "files/cache/";
		
		/* ### Member Vars ### */
			/**
			 * @name DbConn
			 * @access private
			 * @var \PDO
			 */
			private $mDbConn;		
		
		/* ### Constructors ### */
			function __construct(\PDO $aDbConn){
				$this->setDbConn($aDbConn);
			}	
		
		/* ### Public Methods ### */
				public function insertFile($aTmpFileName){
					/*** Create hashes ***/
					 	$hash =md5(current_millis().$aTmpFileName.md5_file(self::cCACHE_FOLDER.$aTmpFileName));
					 /*** check if file allready exists ***/
					 	if($this->fileExists($hash)){
					 		print "file already in db: $aTmpFileName";
					 		return $hash;
					 	}
					 /*** Get Mime Type ***/
					 	$finfo = finfo_open(FILEINFO_MIME_TYPE);
					 	$mime=finfo_file($finfo, self::cCACHE_FOLDER.$aTmpFileName);					 	
					/*** Open tmp file ***/
						$fh = fopen(self::cCACHE_FOLDER.$aTmpFileName,"r");
					/*** prepare sql statement ***/
					 	$stmt = $this->getDbConn()->prepare("INSERT INTO files (hash,file,type,size) VALUES (:hash, :file, :type, :size)");				 
					 	$stmt->bindParam(':file', $fh, \PDO::PARAM_LOB);
					 	$stmt->bindParam(':hash',$hash);
					 	$stmt->bindParam(':type',$mime);
					 	$stmt->bindValue(':size',filesize(self::cCACHE_FOLDER.$aTmpFileName));
					/*** execute ***/
					 	$this->getDbConn()->beginTransaction();
					 	$stmt->execute();
					 	$this->getDbConn()->commit();
					 	
					 return $hash;
				}
		
			
			//----------------------------------------
			
				public function deleteFile($aHash){
					$stmt = $this->getDbConn()->prepare("DELETE FROM files WHERE hash = ?");
					$stmt->execute(Array($aHash));	
					if($stmt->rowCount()==0){
						throw new \Exception("File not found hash:".$aHash);
					}	
					return true;				
				}
			
			
			//----------------------------------------
			
				public function getFile($aHash){
					$stmt = $this->getDbConn()->prepare("SELECT file FROM files WHERE hash = ?");
					$stmt->execute(Array($aHash));
					$stmt->bindColumn(1, $lob, \PDO::PARAM_LOB);
					$stmt->fetch(\PDO::FETCH_BOUND);
					if(is_string($lob)){
						return $lob;
					}elseif(is_null($lob)){
						return false;
					}else{
						return stream_get_contents($lob);					
					}
				}

			//-----------------------------------------
			
				public function updateFile($aHash,$aTmpFileName,$aType){
						$fh = fopen(self::cCACHE_FOLDER.$aTmpFileName,"r");
					/*** prepare sql statement ***/
					 	$stmt = $this->getDbConn()->prepare("UPDATE files SET file = :file, type = :type,size = :size WHERE hash = :hash");				 
					 	$stmt->bindParam(':file', $fh, \PDO::PARAM_LOB);
					 	$stmt->bindParam(':hash',$aHash);
					 	$stmt->bindParam(':type',$aType);
					 	$stmt->bindValue(':size',filesize(self::cCACHE_FOLDER.$aTmpFileName));
					/*** execute ***/
					 	$this->getDbConn()->beginTransaction();
					 	$stmt->execute();
					 	if($this->getDbConn()->commit()){
					 		return true;
					 	}else{
					 		return false;
					 	}					 
				}	

			//------------------------------------------
			
				public function getType($aHash){
					$stmt = $this->getDbConn()->prepare("SELECT type FROM files WHERE hash = ?");
					$stmt->execute(Array($aHash));					
					$re = $stmt->fetch();
					return $re['type'];
				}
				
			//-------------------------------------------
			
				public function getFileSize($aHash){
					$stmt = $this->getDbConn()->prepare("SELECT size FROM files WHERE hash = ?");
					$stmt->execute(Array($aHash));									
					$re = $stmt->fetch();
					return $re['size'];					
				}
		
			//-------------------------------------------
			
				public function fileExists($aFileHash){
					$stmt = $this->getDbConn()->prepare("SELECT COUNT(*) FROM files WHERE hash = ?");
					$stmt->execute(Array($aFileHash));
					if($stmt->fetchColumn()!=false){return true;}else{return false;}
				}
		/* ### Private Methods ### */

		
		/* ### Setters and Getters ### */
			private function setDbConn($aVal){$this->mDbConn = $aVal;}
			private function getDbConn(){return $this->mDbConn;}
	}
	
?>