<?php 
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/

	/* File-desc: Handles the files and folders in the Db */	
	namespace db\fs;
	include_once("db/fs/FileHandler.inc");
	include_once("db/fs/FsEntry.inc");

	class FileStructureHandler{
		/* ### Consts ### */
			const FS_TYPE_FILE = 1;
			const FS_TYPE_FOLDER = 2;
		/* ### Member Vars ### */
			/**
			 * @name DbConn
			 * @access private
			 * @var \PDO
			 */
			private $mDbConn;
			private $mUserId;
			/**
			 * @name mFileHandler
			 * @var FileHandler
			 */
			private $mFileHandler;
		/* ### Constructor ### */
			function __construct(\PDO $aDbConn,$aUserId){
				$this->setDbConn($aDbConn);
				$this->setUserId($aUserId);
				$this->setFileHandler(new FileHandler($this->getDbConn()));
				$this->removeUnlinkedFiles(); //TODO only every 100 executions
			}
		/* ### Public Methods ### */
				public function addFsEntry($aOwner,$aPath,$aName,$aLastChange,$aType,$aHash){
						$upperFolder = $this->getFolderId($aPath);	
					/*** Check for duplicates ***/
						$sqlQuery = "SELECT id FROM filesystem WHERE owner = :owner AND name = :name AND path = :path AND upper_folder = :upperFolder AND type = :type";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt->bindParam(':owner',$aOwner);
						$stmt->bindParam(':name',$aName);
						$stmt->bindParam(':path',$aPath);
						$stmt->bindParam(':upperFolder',$upperFolder);
						$stmt->bindParam(':type',$aType);
						/*** Execute ***/
							$stmt->execute();						
	   						/* Check the number of rows that match the SELECT statement */
	   							if ($stmt->fetchColumn()==0) {					
	   								/*** Add fs entry sql ***/
									$sqlQuery ="INSERT INTO filesystem (owner,name,path,upper_folder,last_change,type,hash,last_update)VALUES(:owner,:name,:path,:upperFolder,:lastChange,:type,:hash,:lastUpdate)";
									$stmt = $this->getDbConn()->prepare($sqlQuery);
									$stmt->bindParam(':owner',$aOwner);
									$stmt->bindParam(':name',$aName);
									$stmt->bindParam(':path',$aPath);
									$stmt->bindParam(':upperFolder',$upperFolder);
									$stmt->bindParam(':hash',$aHash);
									$stmt->bindParam(':type',$aType);
									$stmt->bindParam(':lastChange',$aLastChange);
									$stmt->bindParam(':lastUpdate',$aLastChange);
									$stmt->execute();
	   							}else{
	   								throw new \Exception("Entry already exists!");
	   							}
	   				return true;
				}
			
			
			//-------------------------------------------	
				
				public function removeFsEntry($aPath,$aName){
					/*** Get upper folder ***/
						$upperFolderId = $this->getFolderId($aPath);					
					/*** get entry infos ***/
						$entry = $this->getEntryInfos($aPath,$aName);	
					/*** delete entry ***/
						$this->removeFsEntryById($entry['id']);					
					/*** type specific delete ***/
						if($entry['type']==self::FS_TYPE_FILE){							
							/*** Delete file ***/
								$this->getFileHandler()->deleteFile($entry['hash']);	
								return true;							
						}else{
							/*** Delete Subfolder ***/
								$stmt = $this->getDbConn()->prepare("SELECT name,id,type,path FROM filesystem WHERE upper_folder=:upperFolder AND owner='".$this->getUserId()."'");
								$deleteList = Array(); 
								$this->recDeleteFolderList($deleteList,$stmt,$entry['id']);
							/*** Delete them ***/
								foreach($deleteList as $element){
									/*** Remove fs entry from the database ***/
										$this->removeFsEntryById($element);
								}							
								return true;
						}		
				}				
				
			//-------------------------------------------
			
				public function renameFsEntry($aPath,$aOldName,$aNewName){	
						$oldHash = "";
						$newHash = "";
						/*** Get type ***/
							$oldEntry = $this->getEntryInfos($aPath,$aOldName);				
						/*** Hashes ***/
							if($oldEntry['type'] == self::FS_TYPE_FOLDER){
									$oldHash = $this->makeFolderHash($aPath,$aOldName);
									$newHash = $this->makeFolderHash($aPath,$aNewName);									
							}else{
								$oldHash = $oldEntry['hash'];
								$newHash = $oldHash;
							}
					/*** prepare and execute sql query ***/
						$sqlQuery = "UPDATE filesystem SET name = :name, hash = :hash, last_update = :updateTime WHERE hash = :oldHash AND owner='".$this->getUserId()."'";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt->bindParam(':name',$aNewName);
						$stmt->bindParam(':hash',$newHash);
						$stmt->bindParam(':oldHash',$oldHash);
						$stmt->bindValue(':updateTime',current_millis());
						if($stmt->execute()){
							if($oldEntry['type'] == self::FS_TYPE_FOLDER){
								$this->updatePaths($aPath,$aNewName);			
							}							
							return true;
						}								
				}
				

			//-------------------------------------------
				
				/**
				 * 
				 * @param String $aPath /path/to/folder/
				 * @param String $aName
				 */
				public function getEntryInfos($aPath,$aName){
					/*** Get upper folder ***/
						$upperFolderId = $this->getFolderId($aPath);
					/*** sql query ***/
						$sqlQuery = "SELECT * FROM filesystem WHERE name= :name AND upper_folder = :folder AND owner='".$this->getUserId()."'";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt->bindParam(':name',$aName);
						$stmt->bindParam(':folder',$upperFolderId);
						$stmt->execute();						
						return $stmt->fetch();
				}
				
			
			//------------------------------------

				/**
				 * @param String $aPath /path/to/folder/
				 * @param String $aName
				 * @return boolean
				 */
				public function entryExists($aPath,$aName){
						$sqlQuery = "SELECT id FROM filesystem WHERE name= :name AND path = :path AND owner='".$this->getUserId()."'";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt->bindParam(':path',$aPath);
						$stmt->bindParam(':name',$aName);						
						$stmt->execute();		
						/* Check the number of rows that match the SELECT statement */
						$res = $stmt->fetchAll();
   							if (count($res)>0) {				
   								return true;
   							}else{
   								return false;
   							}
				}
				

			//------------------------------------
				
				
				/**
				 * @param String $aPath /path/to/folder/
				 * @param String $aName Name of Entry
				 * @return long lastChangeDate
				 */
				public function getLastChangeDate($aPath,$aName){
					/*** Get upper folder ***/
						$upperFolderId = $this->getFolderId($aPath);
					/*** sql query ***/
						$sqlQuery = "SELECT last_change FROM filesystem WHERE upper_folder= :folder AND name= :name AND owner='".$this->getUserId()."'";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt = $this->bindParam(':folder',$upperFolderId);
						$stmt = $this->bindParam(':name',$aName);
						$stmt->execute();
						$res = $stmt->fetch();
						return $res['last_change'];
				}
				
				
			//-----------------------------

				public function setLastChangeDate($aPath,$aName,$aDate){
					/*** Get upper folder ***/
					//	$upperFolderId = $this->getFolderId($aPath);
					/*** sql query ***/
						$sqlQuery ="UPDATE filesystem SET last_change = :lastChange, last_update = :lastUpdate WHERE path = :path AND name = :name AND owner='".$this->getUserId()."'";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt->bindParam(':path',$aPath);
						$stmt->bindParam(':lastChange',$aDate);
						$stmt->bindParam(':name',$aName);
						$stmt->bindValue(":lastUpdate",current_millis());
						return $stmt->execute();					
				}
				//-----------------------------
				
			//-----------------------------
				
				/**
				 * @param String $aPath /path/to/folder
				 * @param String $aName entry name
				 * @return long lastUpdateDate
				 */
				public function getLastUdateDate($aPath,$aName){
					/*** Get upper folder ***/
						$upperFolderId = $this->getFolderId($aPath);
					/*** sql query ***/
						$sqlQuery = "SELECT last_update FROM filesystem WHERE upper_folder= :folder AND name= :name AND owner='".$this->getUserId()."'";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt = $this->bindParam(':folder',$upperFolderId);
						$stmt = $this->bindParam(':name',$aName);
						$stmt->execute();
						$res = $stmt->fetch();
						return $res['last_update'];
				}
								
				
			//-------------------------------------------
				
				public function setLastUpdateDate($aPath,$aFileName,$aDate){
					/*** Get upper folder ***/
						$upperFolderId = $this->getFolderId($aPath);
					/*** sql query ***/
						$sqlQuery ="UPDATE filesystem SET last_update = :lastUpdate WHERE upper_folder = :folder AND name = :name AND owner='".$this->getUserId()."'";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt = $this->bindParam(':folder',$upperFolderId);
						$stmt = $this->bindParam(':lastUpdate',$Date,\PDO::PARAM_INT);
						$stmt = $this->bindParam(':name',$aName);
						return $stmt->execute();					
				}
				
			
			//-------------------------------------------
				
				/**
				 * @name getFsList
				 * @param String $aRootFolder /path/to/start/folder/
				 */
				public function getFsList($aRootFolder,$aTime,$aRecursive){					
					$topFolderId = $this->getfolderId($aRootFolder);				
					$arr = Array();
					/*** Add root folder ***/
						$stmt = $this->getDbConn()->prepare("SELECT * FROM filesystem WHERE id = :id AND last_update > :time AND owner = '".$this->getUserId()."'");
						$stmt->bindValue(":id",$topFolderId);
						$stmt->bindValue(":time",$aTime);
						$stmt->execute();
						$obj = $stmt->fetchALL(\PDO::FETCH_CLASS, '\db\Fs\FsEntry');
						foreach($obj as $fsEntry){
							$arr[] = $fsEntry;
						}
											
					/*** Recursive filelist (subfolders)***/						
						$stmt = $this->getDbConn()->prepare("SELECT * FROM filesystem WHERE upper_folder= :upperFolder AND last_update > :time AND owner = '".$this->getUserId()."' ORDER BY name ASC");			
						$stmt->bindParam(":time",$aTime);
						$this->getRecFsList($arr,$topFolderId,$stmt,$aRecursive); //If not recursive get just the 1. deepth
					return $arr;
				}
				
				
			//--------------------------------------------
				
				public function updatePaths($aPath,$aFolderName){
					$id =$this->getFolderId($aPath,$aFolderName);
					$dbStmtSelect = $this->getDbConn()->prepare("SELECT id,type,name FROM filesystem WHERE upper_folder = :upperFolder AND owner='".$this->getUserId()."'");
					$this->updatePathsRec($dbStmtSelect,$id,$aPath.$aFolderName."/");
				}
		
		
			//---------------------------------------------

				public function makeFolderHash($aPath,$aFolderName){								
					$hashVal = $this->getUserId().$aPath.$aFolderName.(($aPath =="" && $aFolderName=="" || $aPath!="" && $aFolderName!="") ? "/":"");											
					return md5($hashVal);
				}
				
			//------------------------------------------------
			
				public function getFolderId($aPath,$aFolderName=""){				
					/*** Generate md5 hash ***/
						$hash = $this->makeFolderHash($aPath,$aFolderName);
					/*** Search and return folder id ***/
						$sql = "SELECT id FROM filesystem WHERE hash= :hash AND type = :type AND owner='".$this->getUserId()."'";
						$stmt = $this->getDbConn()->prepare($sql);
						$stmt->bindParam(':hash',$hash);
						$stmt->bindValue(':type',self::FS_TYPE_FOLDER);			
						if($stmt->execute()){		
							$erg = $stmt->fetch();
							if($erg==null){
								return 0; //Assume root
							}							
							return $erg['id'];
						}	
						return false;				
				}	
		/* ### Private Methods ### */	
					/**
				 * @name recDeleteFolder
				 * @param int $aActFolderId
				 * @param FileHandler $aFileHandler
				 */
				private function recDeleteFolderList(&$aArray,$dbStmt,$aActFolderId){				
					$dbStmt->bindValue(':upperFolder',$aActFolderId);
					$dbStmt->execute();
					foreach($dbStmt->fetchAll() as $entry){					
						switch($entry['type']){
							case self::FS_TYPE_FILE:
									$aArray[]=$entry['id'];
								break;
							case self::FS_TYPE_FOLDER:
									$aArray[]=$entry['id'];
									$this->recDeleteFolderList($aArray,$dbStmt,$entry['id']);									
								break;
							default:
								assert(false);
						}
					}
					return;
				}
				
				
			//-------------------------------------------			
				
				private function updatePathsRec($aDbStmtSelect,$aActFolderId,$aActPath){
					$aDbStmtSelect->bindValue(':upperFolder',$aActFolderId);
					$aDbStmtSelect->execute();
					$reArr=Array();
					foreach($aDbStmtSelect->fetchAll() as $entry){					
						switch($entry['type']){
							case self::FS_TYPE_FILE:
								$dbStmtUpdate= $this->getDbConn()->prepare("UPDATE filesystem SET path=:path, last_update = :updateTime WHERE id = :id AND owner='".$this->getUserId()."'");	
								$dbStmtUpdate->bindValue(':id',$entry['id']);
								$dbStmtUpdate->bindValue(":path",$aActPath);
								$dbStmtUpdate->bindValue(':updateTime',current_millis());
								$dbStmtUpdate->execute();
								break;
							case self::FS_TYPE_FOLDER:
								$this->updatePathsRec($aDbStmtSelect,$entry['id'],$aActPath.$entry['name']."/");
								$dbStmtUpdate= $this->getDbConn()->prepare("UPDATE filesystem SET path=:path, hash = :hash, last_update = :updateTime WHERE id = :id");	
								$dbStmtUpdate->bindValue(':id',$entry['id']);
								$dbStmtUpdate->bindParam(":path",$aActPath);								
								$dbStmtUpdate->bindValue(":hash",$this->makeFolderHash($aActPath,$entry['name']));
								$dbStmtUpdate->bindValue(':updateTime',current_millis());
								$dbStmtUpdate->execute();
								break;
							default:
								assert(false);
						}
					}
				}
					

			//-------------------------------------------

				private function getRecFsList(&$aArray, $aActFolderId,$aStmt,$aRecursive){
					/*** Sql statement ***/
						$aStmt->bindValue(":upperFolder",$aActFolderId);
						$aStmt->execute();
						$obj = $aStmt->fetchALL(\PDO::FETCH_CLASS, '\db\Fs\FsEntry');
					foreach($obj as $fsEntry){
						switch($fsEntry->getType()){
							case self::FS_TYPE_FILE:
								$aArray[] = $fsEntry;
								break;
							case self::FS_TYPE_FOLDER:
								$aArray[] = $fsEntry;
								if($aRecursive==true){
									$this->getRecFsList($aArray,$fsEntry->getId(),$aStmt,$aRecursive);	
								}																				
								break;
							default:
								assert(false);
						}
					};
					return;
				}

			//---------------------------------------------
			
				private function removeFsEntryById($aId){
					/*** delete folder ***/
						$sqlQuery = "DELETE FROM filesystem WHERE id = :id";
						$stmt = $this->getDbConn()->prepare($sqlQuery);
						$stmt->bindParam(':id',$aId);
						if(!$stmt->execute()){return false;}					
				}	

			//----------------------------------------------
			
				private function removeUnlinkedFiles(){
					//TODO implement
				}
				
				
		/* ### Setters and Getters ### */			
			private function setDbConn($aVal){$this->mDbConn = $aVal;}
			private function getDbConn(){return $this->mDbConn;}
			
			private function setFileHandler($aVal){$this->mFileHandler = $aVal;}
			private function getFileHandler(){return $this->mFileHandler;}
			
			private function setUserId($aVal){$this->mUserId = $aVal;}
			private function getUserId(){return $this->mUserId;}
	}
?>