<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/

	/* File-desc: Handles the files and folders in the Db */	

/*** head ***/
	namespace db\fs;
	include_once("db/fs/FileHandler.inc");
	include_once("db/fs/FileStructureHandler.inc");
	include_once("db/history/History.inc");
	use db\History\History;	
/*** class ***/
	class FsHandler{		
		/* ### Member Vars ### */
			/**
			 * @name mDbConn
			 * @access private
			 * @var \PDO
			 */
			private $mDbConn;
			/**
			 * @name FileHandler
			 * @access private
			 * @var FileHandler
			 */
			private $mFileHandler;
			/**
			 * @name FileStructureHandler
			 * @access private
			 * @var FileStructureHandler
			 */
			private $mFileStructureHandler;		

			/**
			 * Action History
			 * @var History
			 */
			private $mHistory;
			
			private $mUserId;
			
		/* ### Constructors ### */
			function __construct(\PDO $aDbConn,$aUserId){
				$this->setDbConn($aDbConn);
				$this->setUserId($aUserId);
				$this->setFileStructureHandler(new FileStructureHandler($this->getDbConn(),$this->getUserId()));
				$this->setFileHandler(new FileHandler($this->getDbConn()));
				$this->setHistory(new History($this->getDbConn(),$this->getUserId()));
			}
		
		/* ### Public Methods ### */
			/**
			 * 
			 * @param String $aRootFolder
			 * @param long $aTime
			 * @param boolean $aRecursive
			 */
			public function getFsList($aRootFolder,$aTime,$aRecursive){
				return $this->getFileStructureHandler()->getFslist($aRootFolder,$aTime,$aRecursive);
			}
		
			
			//########## Folder operations ###########//			
				/**
				 * @name addFolder
				 * @param String $aPath /path/to/folder
				 * @param String $aFolderName FolderName
				 * @param int $aLastChange
				 * @param int [$aUserId=-1]
				 * @return boolean
				 */			
				public function addFolder($aPath,$aFolderName,$aLastChange=-1){	
					if($aLastChange=-1){$aLastChange=current_millis();}				
						$hash = $this->getFileStructureHandler()->makeFolderHash($aPath,$aFolderName);
					/*** Insert folder into the filestructure ***/
						if($this->getFileStructureHandler()->addFsEntry($this->getUserId(),$aPath, $aFolderName,$aLastChange,FileStructureHandler::FS_TYPE_FOLDER,$hash)){
							/** Add History entry **/
								$this->getHistory()->addHistoryEntry(History::cTYPE_ADDED_FOLDER,$aPath.$aFolderName);
							return true;
						}else{
							return false;
						}
				}
			

			//##############################################//
			
			//################# File methods ##############//					
					/**
					 * @name addFile
					 * @param String $aPath path/to/folder/
					 * @param String $aFileName FileName
					 * @param String $aFileHash md5 hash of the connected file
					 * @return boolean
					 */
					public function addFile($aPath,$aName,$aTmpName,$aLastChange=0,$aDeleteFileAfterUpload=true,$aIsInCache=false,$aExistingFileHash=""){
						if ($aLastChange==0){$aLastChange=current_millis();}
							/*** Move file from tmp to the cache folder ***/
								$tmpSavePath =FileHandler::cCACHE_FOLDER.$aTmpName;
								if ($aIsInCache==false) {								
									if(!move_uploaded_file("/tmp/".$aTmpName,$tmpSavePath)){
										return false;
									}
								}									
							/*** Add fs entry ***/ 													
								if($this->getFileStructureHandler()->addFsEntry($this->getUserId(),$aPath,$aName,$aLastChange,FileStructureHandler::FS_TYPE_FILE,$aTmpName != null ? $this->getFileHandler()->insertFile($aTmpName):$aExistingFileHash)){
									/** Delete chache file **/
										if($aDeleteFileAfterUpload==true){
											unlink($tmpSavePath);
										}
									/** Add History entry **/
										$this->getHistory()->addHistoryEntry(History::cTYPE_ADDED_FILE,$aPath.$aName);
									return true;
								}else{
									return false;
								}				
					}
					
				
				//--------------------------------
					
					public function getFileHash($aPath,$aFileName){
						$res = $this->getFileStructureHandler()->getEntryInfos($aPath,$aFileName);
						return $res['hash'];
					}

				//--------------------------------

					public function fileExists($aHash){
						return $this->getFileHandler()->fileHashExists($aHash);
					}
					
				//--------------------------------
			
					public function getFile($aPath,$aFileName){
						/*** Get the files hash ***/
							$stmt = $this->getDbConn()->prepare("SELECT hash FROM filesystem WHERE path = ? AND name = ?");
							$stmt->execute(Array($aPath,$aFileName));
							$res = $stmt->fetch();
						return $this->getFileHandler()->getFile($res['hash']);				
					}
			
			
				//--------------------------------
				
					public function updateFile($aPath,$aName,$aTmpName,$aType,$aLastChange = 0,$aDeleteFileAfterUpload=true){
						if ($aLastChange==0){$aLastChange=current_millis();}
							/*** Move file from tmp to the cache folder ***/
								$tmpSavePath =FileHandler::cCACHE_FOLDER.$aTmpName;							
									if(!move_uploaded_file("/tmp/".$aTmpName,$tmpSavePath)){
										echo "fileupload error!";
										return false;
									}
							/*** Update file ***/																					
								if($this->getFileHandler()->updateFile($this->getFileHash($aPath,$aName),$aTmpName,$aType)){
									/*** Delete chache file ***/
										if($aDeleteFileAfterUpload==true){
											unlink($tmpSavePath);
										}
									/*** Set lastchange ***/			
										$this->getFileStructureHandler()->setLastChangeDate($aPath,$aName,$aLastChange);
										$this->getHistory()->addHistoryEntry(History::cTYPE_MODIFIED,$aPath.$aFolderName);
										return true;
									}else{
										return false;
									}	
					}
			
			
			//############## Other methods ###################\\				
					public function createNewUserFs(){
							$this->addFolder("","",current_millis()) ; //Root folder
						/*** Add sample folders ***/
							$this->addFolder("/","Docs",current_millis());
							$this->addFolder("/Docs/","alpha",current_millis());
							$this->addFolder("/","Samples",current_millis());
							$this->addFile("/Samples/","Readme.txt","Readme.txt",current_millis(),false,true);
					}
		
				
				//--------------------------------
				
					/**
					 * @return int Default max user space
					 */
					public function getDefaultUserSpace(){
						$sql = "SELECT default_max_user_space FROM config";
						$res = $this->getDbConn()->query($sql);
						$info = $res->fetch();
						return $info['default_max_user_space'];
				}

			
			//################# main functions ##############\\			
					public function setLastChange($aPath,$aName,$aLastChange){
						if($this->getFileStructureHandler()->setLastChangeDate($aPath,$aName,$aLastChange)){
							$this->getHistory()->addHistoryEntry(History::cTYPE_MODIFIED,$aPath.$aFolderName);
						}	
					}
				
				
				//--------------------------------
				
					public function deleteFsEntry($aPath,$aName){						
						if($this->getFileStructureHandler()->removeFsEntry($aPath,$aName)){
							/** Add History entry **/
							$this->getHistory()->addHistoryEntry(History::cTYPE_DELETED,$aPath.$aName);
						}
					}
			
			
				//--------------------------------
				
					public function renameFsEntry($aPath,$aOldName,$aNewName){
						/** Add History entry **/
						if($this->getFileStructureHandler()->renameFsEntry($aPath,$aOldName,$aNewName)){
							$this->getHistory()->addHistoryEntry(History::cTYPE_RENAMED,$aPath.$aOldName."=>".$aPath.$aNewName);							
						}	
					}
			
				
				//---------------------------------
					public function isFile($aPath,$aName){
						$entry = $this->getFileStructureHandler()->getEntryInfos($aPath,$aName);
						if($entry['type']==FileStructureHandler::FS_TYPE_FILE){
							return true;
						}else{
							return false;
						}
					}
				
				//----------------------------------
				
					public function getEntry($aPath,$aName){
						$details =  $this->getFileStructureHandler()->getEntryInfos($aPath, $aName);
						if($details==false){return false;}
						$entry = new FsEntry();
						$entry->setValues($details['id'], $details['owner'],$details['name'], $details['upper_folder'], $details['last_change'], $details['type'], $details['hash'],$details['path']);
						return $entry;						
					}
		/* ### Private methods ### */
	
				
		
		/* ### Setters and Getters ### */
			private function setDbConn($aMod){$this->mDbConn = $aMod;}
			private function getDbConn(){return $this->mDbConn;}
			
			function setUserId($aUserId){$this->mUserId=$aUserId;}
			function getUserId(){return $this->mUserId;}
			
			private function setFileStructureHandler(FileStructureHandler $aMod){$this->mFileStructureHandler = $aMod;}
			private function getFileStructureHandler(){return $this->mFileStructureHandler ;}
		
			private function setFileHandler(FileHandler $aMod){$this->mFileHandler = $aMod;}
			private function getFileHandler(){return $this->mFileHandler ;}
			
			private function setHistory(History $aMod){$this->mHistory = $aMod;}
			private function getHistory(){return $this->mHistory;}
			
	}
	
?>