<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/

	/* File-desc: Shows the WebInterface */


	namespace webInterface;	

	include_once "webInterface/internPage/InternPage.inc";
	include_once "webInterface/internPage/AjaxHandler.inc";
	include_once "webInterface/internPage/pageModules/JqueryUiDesignLoader.inc";

	use webInterface\internPage\AjaxHandler;
	use webInterface\internPage\InternPage;
	
	class WebInterface{
		/* ### Member Vars ### */
			private $mDbConn;
			private $mNoticer;
		
		
		/* ### Constructor ### */
			function __construct($aDbConn){
				$this->setDbConn($aDbConn);
				if(isset($_SESSION['userId'])){
					$this->setUserId($_SESSION['userId']);
				}else{
					$this->setUserId(-1);
				}
				$this->show();
			}	
		
		/* ### Public Methods ### */
			public function show(){
				$this->handleRequests();
				if(!$this->isLoggedIn()){
					echo $this->showLoginPage();
				}else{
					if(isset($_GET['ajax']) && $_GET['ajax']=="true"){
						$this->ajax();	
					}else{
						echo $this->showInternPage();
					}
				}
			}
		
		/* ### Private Methods ### */

			private function handleRequests(){
				if(isset($_POST['login'])){
					if(!$this->checkLogin($_POST['username'],$_POST['pwd'])){
						// TODO ADD Noticer message
					}
				}elseif(isset($_GET['logout'])){
					$this->logout();
				}
			}
			
			//------------------------------------
			
			private function checkLogin($aUsername,$aPwd){
				/* *** generate md5 pwd ***/
					$aPwd = md5($aPwd);
				/* *** prepare sql query ***/
					$sqlQuery = "SELECT * FROM users WHERE name= :name AND password= :pwd";
					$stmt = $this->getDbConn()->prepare($sqlQuery);
					$stmt->bindParam(':name',$aUsername);
					$stmt->bindParam(':pwd',$aPwd);
				/* *** execute *** */
					$stmt->execute();
					$res = $stmt->fetch();
					if(is_array($res)){
						/*** Set session vars ***/
						$_SESSION['loggedIn']=true;
						$_SESSION['userId']=$res['id'];
						$this->setUserId($res['id']);
						$_SESSION['isAdmin']=$res['admin'];						
						return true;
					}
			}
			
			//------------------------------
			
			private function logout(){
				$_SESSION['loggedIn']=null;
				$_SESSION['userId']=null;
				$_SESSION['isAdmin']=null;					
				header("Location: ".$_SERVER['PHP_SELF']);
				exit; 
			}
			
			//------------------------------
			
			private function isLoggedIn(){
				if(isset($_SESSION['loggedIn']) && $_SESSION['loggedIn']==true){return true;}else{return false;}
			}
			
			//------------------------------
			
			private function isAdmin(){
				if(isset($_SESSION['isAdmin']) &&$_SESSION['isAdmin']==true){return true;}else{return false;}
			}
			
			//------------------------------
			
			private function showLoginPage(){
				return file_get_contents("templates/login/login.html");
			}
		
			//-------------------------------
			
			private function showInternPage(){
				$ip = new InternPage($this->getDbConn(),$this->getUserId());
				return $ip->show();
			}
			
			//------------------------------
			
			private function ajax(){
				$ajaxHandler = new AjaxHandler($this->getDbConn(),$this->getUserId());
				$ajaxHandler->work();
			}
		
		/* ### Setters and Getters ### */
			private function setDbConn($aMod){$this->mDbConn = $aMod;}
			private function getDbConn(){return $this->mDbConn;}	
			private function setUserId($aId){$this->mUserId = $aId;}
			private function getUserId(){return $this->mUserId;}
	}

?>