<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/
	
	namespace webInterface\internPage;
	
	use webInterface\internPage\ajaxModules\HistoryControler;
	use webInterface\internPage\ajaxModules\FileSystemControler;
	use db\fs\FsHandler;
	use db\fs\FileHandler;
	
	include_once("db/fs/FsHandler.inc");	
	include_once("db/fs/FileHandler.inc");
	
	include_once("webInterface/internPage/ajaxModules/FileSystemControler.inc");
	include_once("webInterface/internPage/ajaxModules/HistoryControler.inc");
	
	class AjaxHandler{
		/* ### Member Vars ### */
			/**
			 * Database connection
			 * @var \PDO
			 */
			private $mDbConn;
		
			
			private $mUserId;
		
		/* ### Constructor ### */
			
			/**
			 * AjaxHandler
			 * @param \PDO $aDbConn
			 */
			function __construct($aDbConn,$aUserId){
				$this->setDbConn($aDbConn);
				$this->setUserId($aUserId);
			}
		/* ### Public Methods ### */
			public function work(){
				if(isset($_GET['request'])){
					switch($_GET['request']){
						case "getMimeType":
							echo $this->getMimeType($_GET['filepath'],$_GET['filename']);
							break;							
						case "getFileSystemBrowser":
							echo $this->getFileSystemList();
							break;
						case "getHistoryViewer":
							echo $this->getHistory(20);
							break;
						case "deleteFsEntry":						
							if(isset($_GET['path']) && isset($_GET['name'])){
								$this->deleteFsEntry($_GET['path'],$_GET['name']);
							}
							break;
						case "renameFsEntry":
							if(isset($_GET['path']) && isset($_GET['name']) && isset($_GET['newName'])){								
								$this->renameFsEntry($_GET['path'],$_GET['name'],$_GET['newName']);
							}
							break;
						default:
							echo "unknown request: ".$_GET['request'];
					}					
				}
			}
		
		/* ### Private Methods ### */
			private function getMimeType($aPath,$aName){
				$fsHandler = new FsHandler($this->getDbConn(),$this->getUserId());
				$fileHandler = new FileHandler($this->getDbConn());
				$entry = $fsHandler->getEntry($aPath, $aName);
				if($entry === false){
					echo "File not found: ".$aPath. " Name: ".$aName;
					die();
				}
				$mime = $fileHandler->getType($entry->getHash());
				return $mime;
			}
			
			//-----------------------------------------------
			
			private function getFileSystemList(){
				$fsc = new FileSystemControler($this->getDbConn(),$this->getUserId());
				return $fsc->getFileSystemList();
			}
			
			//------------------------------------------------
			
			private function getHistory($aEntryCount){
				$history = new HistoryControler($this->getDbConn(), $this->getUserId());
				return $history->getRecentHistoryList($aEntryCount);
			}
		
			//--------------------------------------------------
			
			private function deleteFsEntry($aPath,$aName){
				$fsHandler = new FsHandler($this->getDbConn(),$this->getUserId());					
				$fsHandler->deleteFsEntry($aPath,$aName);
			}
		
			//--------------------------------------------------
			
			private function renameFsEntry($aPath,$aName,$aNewName){
				$fsHandler = new FsHandler($this->getDbConn(),$this->getUserId());
				$fsHandler->renameFsEntry($aPath,$aName, $aNewName);
			}
		/* ### Setter and Getter ### */
			private function setDbConn($aMod){$this->mDbConn = $aMod;}
			private function getDbConn(){return $this->mDbConn;}
			
			private function setUserId($aId){$this->mUserId = $aId;}
			private function getUserId(){return $this->mUserId;}
		
	}

?>