<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/

/* File-desc: User Interface controler for the webinterface */
	namespace webInterface\internPage\ajaxModules;	
	
	use db\fs\FileStructureHandler;
	use db\fs\FsHandler;
	use db\fs\FileHandler;
	
	include_once "db/fs/FsHandler.inc";	
	include_once "db/fs/FileHandler.inc";
	include_once "db/fs/FileStructureHandler.inc";
	include_once "webInterface/internPage/ajaxModules/AjaxModule.inc";
	
	class FileSystemControler extends AjaxModule{
		/* ### Consts ### */
			private static $cTimeFormat = "F j, Y, g:i a"; //look @ http://de3.php.net/manual/de/function.date.php
		
		/* ### Member Vars ### */		
		/* ### Constructor ### */
	
		/* ### Public Methods ### */
			public function getFileSystemList(){
				return $this->getFsList();
			}
		/* ### Private Methods ### */
			private function getFsList(){
				$template = file_get_contents("templates/intern/FileSystemControl/fsc.html");
				$searchArr=Array("[filename_txt]","[filesize_txt]","[filechanged_txt]","[filelist]");
				/** get the path **/
					$path = "";
					if(isset($_GET['path'])){
						$path = $_GET['path'];
					}		
				/** Path == file => download it **/
				$replaceArr=Array("Filename","Size","Modified",$this->getFileList($path));				
				return $this->search_replace($searchArr,$replaceArr, $template);
			}
			
			//---------------------------------
			
			private function getFileList($rootFolder){
				$fsHandler = new FsHandler($this->getDbConn(),$this->getUserId());
				$list = $fsHandler->getFsList($rootFolder,0,false);
				$template = file_get_contents("templates/intern/FileSystemControl/fsc_filelist_entry.html");
				$re ="";	
				if(basename($rootFolder,"/")!=""){
					$re.=$this->getUpperLink($rootFolder);
				}
				
				foreach($list as $entry){
					if($entry->getName()!=basename($rootFolder,"/")){ //not root or root folder						
						$size = ($entry->getType()==FileStructureHandler::FS_TYPE_FILE ? $this->getFileSizeText($this->getFileSize($entry)):"");						
						$searchArr=Array("[fileicon]","[filename]","[filesize]","[filechange]","[filelist_entry_id]","[file_root_path]","[filepath]","[filetype]","[buttons_display]");
						$replaceArr=Array($this->getIconTemplate($entry),$entry->getName(),$size,$this->getTimeFromStemp($entry->getLastChange()),$entry->getId(),$entry->getPath(),$entry->getPath().$entry->getName()."/",$entry->getType(),"block");				
						$re.= $this->search_replace($searchArr,$replaceArr, $template);				
					}
				}				
				return $re;
			}
			
			//-------------------------------------
			
			private function getFileSize($entry){
				$fileHandler = new FileHandler($this->getDbConn());
				$size = $fileHandler->getFileSize($entry->getHash());
				return $size;
			}
			
			//--------------------------------------
			
			/** From http://php.net/manual/de/function.filesize.php **/
			private function getFileSizeText($bytes){
				   if ($bytes < 1024) return $bytes.' B';
				   elseif ($bytes < 1048576) return round($bytes / 1024, 2).' KB';
				   elseif ($bytes < 1073741824) return round($bytes / 1048576, 2).' MB';
				   elseif ($bytes < 1099511627776) return round($bytes / 1073741824, 2).' GB';
				   else return round($bytes / 1099511627776, 2).' TB';
			}
			
			//-----------------------------------------
			
			private function getTimeFromStemp($aStamp){				
				return date(self::$cTimeFormat,$aStamp);
			}
			
			//-------------------------------------
			
			private function getIconTemplate($entry,$upperFolder=false){
				if($upperFolder){
					return file_get_contents("templates/intern/FileSystemControl/icon_up.html");
				}
				
				if($entry->getType()==FileStructureHandler::FS_TYPE_FILE){
					return file_get_contents("templates/intern/FileSystemControl/icon_file.html");
				}elseif($entry->getType()==FileStructureHandler::FS_TYPE_FOLDER){
					return file_get_contents("templates/intern/FileSystemControl/icon_folder.html");
				}
			}
			
			//---------------------------------------
			
			private function getUpperLink($aPath){
				if(dirname($aPath) == "/"){$aPath="";}
				$template = file_get_contents("templates/intern/FileSystemControl/fsc_filelist_entry.html");				
				$searchArr=Array("[fileicon]","[filename]","[filesize]","[filechange]","[filelist_entry_id]","[file_root_path]","[filepath]","[filetype]","[buttons_display]");
				$replaceArr=Array($this->getIconTemplate("",true),"Folder up","","","","",dirname($aPath)."/","-1","none");				
				return $this->search_replace($searchArr,$replaceArr, $template);
			}
			
		/* ### Setter and Getter ### */
	}
?>