<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/
	namespace webInterface\internPage\ajaxModules;
	
	use db\History\History;

	include_once "db/history/History.inc";
	include_once "webInterface/internPage/ajaxModules/AjaxModule.inc";
	
	class HistoryControler extends  AjaxModule{
		/* ### Consts ### */
			private static $cTimeFormat = "F j, Y, g:i a"; //look @ http://de3.php.net/manual/de/function.date.php
		/* ### Member Vars ### */
			private $mHistory;
			private $mIconTemplateCache;
		/* ### Constructor ### */
			function __construct($aDbConn,$aUserId){
				parent::__construct($aDbConn,$aUserId);
				$this->setHistory(new History($this->getDbConn(), $this->getUserId()));
				$this->initIconTemplateCache();	
			}
		/* ### Public Methods ### */		
			public function getRecentHistoryList($aEntries){
				$template = file_get_contents("templates/intern/history/history.html");				
				/** Create entries list **/
					$entries = $this->getHistory()->getHistory($aEntries);
					$entry_list="";
					foreach($entries as $entry){
						if($entry->getMsg()!=""){
							$tmpl = file_get_contents("templates/intern/history/entry.html");
							$searchArr=Array("[entry_icon]","[entry_msg]","[entry_time]");
							$replaceArr=Array($this->getIconTemplate($entry->getType()),$this->getMsgText($entry->getType(),$entry->getMsg()),$this->getTimeFromStemp($entry->getTime()));				
							$entry_list.= $this->search_replace($searchArr,$replaceArr, $tmpl);	
						}	
					}				
				return str_replace("[recent_history_entries]",$entry_list, $template);				
			}
		/* ### Private Methods ### */	
			
			private function getMsgText($aType,$aMsg){
				if($aType != History::cTYPE_RENAMED){
					$fsname = basename($aMsg,"/");
					$path = dirname($aMsg);
					if($path == "" || $path=="/"){$path = "Home";}
				}else{
					$parts = explode("=>",$aMsg);
					$fsnameOld = basename($parts[0],"/");
					$pathOld = dirname($parts[0]);
					$fsnameNew = basename($parts[1],"/");
					$pathNew = dirname($parts[1]);					
				}
				 
				switch($aType){
					case History::cTYPE_ADDED_FILE:
						return "Added file <span class='history_entry_fsname'>$fsname</span> in <span class='history_entry_fspath'>$path</span>";
						break;					
					case History::cTYPE_ADDED_FOLDER:
						return "Added folder <span class='history_entry_fsname'>$fsname</span> in <span class='history_entry_fspath'>$path</span>";
						break;
					case History::cTYPE_DELETED:
						return "Deleted <span class='history_entry_fsname'>$fsname</span> in <span class='history_entry_fspath'>$path</span>";
						break;
					case History::cTYPE_MODIFIED:
						return "Modified <span class='history_entry_fsname'>$fsname</span> in <span class='history_entry_fspath'>$path</span>";
						break;
					case History::cTYPE_RENAMED:
						return "Moved <span class='history_entry_fsname'>$fsnameOld</span> from <span class='history_entry_fspath'>$pathOld</span> to <span class='history_entry_fsname'>$fsnameNew</span> in <span class='history_entry_fspath'>$pathNew</span>";
						break;
				}
			}
			
			//-------------------------------------------------
			
			private function getIconTemplate($aType){
				$cache = $this->getIconTemplateCache();				
				return $cache[$aType];
			}
			
			//-----------------------------------------------
			
			private function initIconTemplateCache(){
				$cache =& $this->getIconTemplateCache();				
				$cache[History::cTYPE_ADDED_FILE]=file_get_contents("templates/intern/history/icon_template_added_file.html");
				$cache[History::cTYPE_ADDED_FOLDER]=file_get_contents("templates/intern/history/icon_template_added_folder.html");
				$cache[History::cTYPE_DELETED]=file_get_contents("templates/intern/history/icon_template_deleted.html");
				$cache[History::cTYPE_MODIFIED]=file_get_contents("templates/intern/history/icon_template_modified.html");
				$cache[History::cTYPE_RENAMED]=file_get_contents("templates/intern/history/icon_template_renamed.html");
			}
			
			//-----------------------------------------------
			
			private function getTimeFromStemp($aStamp){				
				return date(self::$cTimeFormat,$aStamp);
			}
			
		/* ### Getter and Setter ### */
			private function setHistory(History $aHistory){$this->mHistory = $aHistory ;}
			private function getHistory(){return $this->mHistory;}
			
			private function setIconTemplateCache($aCache){$this->mIconTemplateCache = $aCache;}
			private function &getIconTemplateCache(){return  $this->mIconTemplateCache;}
	}
?>