<?php
	/* Project: eversync
	 * This file is part of eversync.
		
		eversync is free software: you can redistribute it and/or modify
	    it under the terms of the GNU General Public License as published by
	    the Free Software Foundation, either version 3 of the License, or
	    (at your option) any later version.
	
	    eversync is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with eversync.  If not, see <http://www.gnu.org/licenses/>.
	*/


namespace webInterface\internPage\pageModules;


include_once "db/fs/FsHandler.inc";
include_once "db/fs/FileHandler.inc";
include_once "db/fs/FileStructureHandler.inc";
include_once "webInterface/internPage/pageModules/PageModule.inc";
include_once "db/fs/FsEntry.inc";
use db\fs\FsHandler;
use db\fs\FileHandler;
use db\fs\FileStructureHandler;	


class FolderDownloader extends PageModule{		
	
	/* ### Member Vars ### */
		private $mFsHandler;
		private $mFileHandler;
	/* ### Constructor ### */
		function __construct($aDbConn){
			parent::__construct($aDbConn);
			$this->setFsHandler(new FsHandler($aDbConn, $this->getUserId()));
			$this->setFileHandler(new FileHandler($aDbConn));
		}
		
	/* ### Public Methods ### */
	
		public function folderDownload($aPath,$aName){
			$fsHandler = new FsHandler($this->getDbConn(),$this->getUserId());
			$fileHandler = new FileHandler($this->getDbConn());
			
			/** Create zip **/
				$name = tempnam("tmp/","zip");
				$zipArchive = new \ZipArchive;
				$res = $zipArchive->open($name, \ZipArchive::CREATE);
			/** Add Files and Folders **/
				$this->addFsStructure($zipArchive,$aPath.$aName);
			/** Close zip archive **/	
				$zipArchive->close();
			/** Output **/	
				header("Content-Type: application/zip");
				header("Content-Disposition: attachment; filename=\"$aName.zip\""); 
				echo file_get_contents($name);
		}
		
	/* ### Private Methods ### */
		public function addFsStructure($aZipArchive,$aRootFolder){			
			$aZipArchive->addEmptyDir(basename($aRootFolder,"/"));
			$fs = $this->getFsHandler()->getFsList($aRootFolder."/",0,false);
			foreach($fs as $entry){
				if($entry->getPath().$entry->getName() !=$aRootFolder){ //Without root folder
					if($entry->getType()==FileStructureHandler::FS_TYPE_FILE){
						$aZipArchive->addFromString($entry->getPath().$entry->getName(),$this->getFileHandler()->getFile($entry->getHash()));
					}else{
						$this->addFsStructure($aZipArchive, $entry->getPath().$entry->getName());
					}
				}
			}
		}
	/* ### Setter and Getter ### */
		private function setFsHandler($aHandler){$this->mFsHandler = $aHandler;}
		private function getFsHandler(){return $this->mFsHandler;}
		
		private function setFileHandler($aHandler){$this->mFileHandler = $aHandler;}
		private function getFileHandler(){return $this->mFileHandler;}
}